@@ -21,6 +21,9 @@
 /***************************************************************************/
 
 #include "../common/common.h"
+#include <sys/time.h>
+#include "../ui/xmftp.h"
+#include "../ui/operations.h"
 
 #if defined(__unix__)
 #include <unistd.h>
@@ -81,6 +84,7 @@ struct NetBuf {
 };
 
 pid_t xfer_pid;
+extern void *to_maincb;
 
 /*
 static char *version =
@@ -966,6 +970,7 @@ GLOBALDEF void FtpQuit(netbuf *nControl)
     free(nControl);
 }
 
+#ifdef BAD_GETHOSTBYNAME
 int my_get_host_by_name(char *host, struct in_addr *site)
 {
   FILE *fp;
@@ -1030,6 +1035,7 @@ int my_get_host_by_name(char *host, struct in_addr *site)
 
   return(inet_aton(addr, site));
 }
+#endif
   
 int FtpSiteFP(char *cmd, netbuf *nControl, FILE *fp)
 {
@@ -1188,3 +1194,66 @@ int FtpLoginFP(char *user, char *pass, netbuf *nControl, FILE *fp)
     sprintf(tempbuf,"pass %s",pass);
     return FtpSendCmdFP(tempbuf,'2',nControl, fp);
 }
+
+/*
+int myFtpDir(char *outputfile, char *path, netbuf *nControl)
+{
+    int sData,l;
+    char buf[256];
+    char *dbuf;
+    FILE *output;
+    int siggo = 1;
+    fd_set rfds;
+    struct timeval tv;
+    int retval;
+
+    if (!FtpSendCmd("type A",'2', nControl))
+	return 0;
+    if ((sData = FtpPort(nControl)) == -1)
+	return 0;
+    strcpy(buf,"list");
+    if (path != NULL)
+	sprintf(buf+strlen(buf)," %s",path);
+    if (!FtpSendCmd(buf,'1', nControl))
+	return 0;
+    if (outputfile == NULL)
+	output = stdout;
+    else
+    {
+	output = fopen(outputfile,"w");
+	if (output == NULL)
+	{
+	    perror(outputfile);
+	    output = stdout;
+	}
+    }
+    dbuf = malloc(FTP_BUFSIZ);
+    
+    while(siggo) {
+      FD_ZERO(&rfds);
+      FD_SET(sData, &rfds);
+      tv.tv_sec = 1;
+      tv.tv_usec = 0;
+      
+      retval = select(sData + 1, &rfds, NULL, NULL, &tv);      
+      if(retval > 0) {
+	if(FD_ISSET(sData, &rfds)) {
+	  if((l = net_read(sData,dbuf,FTP_BUFSIZ)) > 0) {
+	    fwrite(dbuf,1,l,output);
+	  }
+	  if(!l)
+	    break;
+	}
+      }
+      else
+	if(retval < 0)
+	  break;
+      InterruptSigGoUI(to_maincb, &siggo);
+    }
+    if (outputfile != NULL)
+	fclose(output);
+    free(dbuf);
+    shutdown(sData,2);
+    net_close(sData);
+}
+*/
