@@ -29,7 +29,7 @@
 /* Used to pass information between the various dialogs being used
  * for deletion
  */
-struct {
+typedef struct {
   FileList *files;           /* List of all files in current section */
   int num_files;             /* Number of files in array */
   int *position_list;        /* Array of positions selected from array */
@@ -38,7 +38,7 @@ struct {
   Main_CB *main_cb;
   int cancel_deletion;       /* Variable becomes 1 when user wants to stop */
   int remote;                /* used to determine if remote or local section */
-} deletion_files;
+} DelFType;
 
 /***************************************************************************/
 
@@ -46,10 +46,6 @@ struct {
 void process_site_command(Widget w, XtPointer client_data, 
 			  XtPointer call_data);
 
-/* Cancel button callback */
-void button_pre_cancel_deletion(Widget w, XtPointer client_data, 
-				XtPointer call_data);
-
 /* Modifies the global variable deletion_files to reflect that the user
  * wants to cancel the deletion
  */
@@ -59,7 +55,7 @@ void stop_deletion(Widget w, XtPointer client_data, XtPointer call_data);
  * from the proper section.
  * returns 1 on success, 0 on failure 
  */
-int delete_subdir(char *filename);
+int delete_subdir(char *filename, DelFType *dfs);
 
 /* Actual performing of deletion (ok button callback) */
 void perform_deletion(Widget w, XtPointer client_data, XtPointer call_data);
@@ -88,7 +84,7 @@ void cancel_deletion(Widget w, XtPointer client_data, XtPointer call_data);
 /* Used to update in real-time the info dialog showing information on what
  * is being deleted.
  */
-void update_delete_dialog(char *filename, int dir);
+void update_delete_dialog(char *filename, int dir, DelFType *dfs);
 
 /* Common deletion function for remote or local */
 void delete(Main_CB *main_cb, int remote);
@@ -96,9 +92,9 @@ void delete(Main_CB *main_cb, int remote);
 /* These functions do the specified operations by determining which
  * section the operation is to be performed on (via the global structure)
  */
-int section_remove_dir(char *filename);
-int section_delete_file(char *filename);
-int section_change_dir(char *filename);
+int section_change_dir(char *filename, DelFType *dfs);
+int section_remove_dir(char *filename, DelFType *dfs);
+int section_delete_file(char *filename, DelFType *dfs);
 
 /***************************************************************************/
 
@@ -120,12 +116,12 @@ void delete_local(Widget w, XtPointer client_data, XtPointer call_data)
 
 void delete(Main_CB *main_cb, int remote)
 {
-  Widget dialog;
   Widget w = main_cb->main_window;
-  char templine[LINE_MAX];
+  char templine[80];
   DirSection section = remote ? main_cb->remote_section : 
     main_cb->local_section;
   int *position_list, position_count;
+  DelFType *deletion_files;
 
   /* Make sure user does not select . or .. for deletion */
   if((XmListPosSelected(section.dir_list, 1) == True) ||
@@ -142,65 +138,64 @@ void delete(Main_CB *main_cb, int remote)
   }
   
   /* setup the global variable with information */
-  deletion_files.files = section.files;
-  deletion_files.num_files = section.num_files;
-  deletion_files.position_list = position_list;
-  deletion_files.position_count = position_count;
-  deletion_files.main_cb = main_cb;
-  deletion_files.cancel_deletion = 0;
-  deletion_files.remote = remote;
+  deletion_files = (DelFType *) malloc(sizeof(DelFType));
+
+  deletion_files->files = section.files;
+  deletion_files->num_files = section.num_files;
+  deletion_files->position_list = position_list;
+  deletion_files->position_count = position_count;
+  deletion_files->main_cb = main_cb;
+  deletion_files->cancel_deletion = 0;
+  deletion_files->remote = remote;
 
   sprintf(templine, "Are you sure you want to delete the %d selected "
 	  "%s files?",
 	  position_count, remote ? "remote" : "local");
 
-  dialog = warning_y_n_dialog(main_cb->main_window, templine);
+  deletion_files->dialog = warning_y_n_dialog(main_cb->main_window, templine);
 
-  XtAddCallback(XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON), 
-		XmNactivateCallback, ok_deletion, main_cb);
-  XtAddCallback(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON), 
-		XmNactivateCallback, cancel_deletion, main_cb);
-
-  XtManageChild(dialog);
+  XtAddCallback(XmMessageBoxGetChild(deletion_files->dialog, 
+				     XmDIALOG_OK_BUTTON), 
+		XmNactivateCallback, ok_deletion, deletion_files);
+  XtAddCallback(XmMessageBoxGetChild(deletion_files->dialog, 
+				     XmDIALOG_CANCEL_BUTTON), 
+		XmNactivateCallback, cancel_deletion, deletion_files);
+  
+  XtManageChild(deletion_files->dialog);
 }
 /***************************************************************************/
   
 void cancel_deletion(Widget w, XtPointer client_data, XtPointer call_data)
 {
-  /* Give back some memory */
-  free(deletion_files.position_list);
-}
-/***************************************************************************/
+  DelFType *dfs = (DelFType *) client_data;
 
-void button_pre_cancel_deletion(Widget w, XtPointer client_data, 
-				XtPointer call_data)
-{
-  Widget dialog = (Widget) client_data;
-
-  cancel_deletion(w, client_data, call_data);
-  XtDestroyWidget(dialog);
+  XtDestroyWidget(dfs->dialog);
+  free(dfs->position_list);
+  free(dfs);
 }
 /***************************************************************************/
 
 void ok_deletion(Widget w, XtPointer client_data, XtPointer call_data)
 {
-  Arg args[10];
+  Arg args[2];
   int n = 0;
-  Main_CB *main_cb = (Main_CB *) client_data;
-  Widget dialog, file_list, rowcol, dir_list, buttons, ok, cancel;
+  DelFType *deletion_files = (DelFType *) client_data;
+  Main_CB *main_cb;
+  Widget file_list, rowcol, dir_list, buttons, ok, cancel;
   int i, index;
   char *filename;
   XmString file;
-
-  dialog = XmCreateFormDialog(main_cb->main_window, "Delete", NULL, 0);
-  deletion_files.dialog = dialog;
-
-  XtVaSetValues(dialog,
+  
+  XtDestroyWidget(deletion_files->dialog);
+  main_cb = deletion_files->main_cb;
+  deletion_files->dialog = XmCreateFormDialog(main_cb->main_window, 
+					      "Delete", NULL, 0);
+  XtVaSetValues(deletion_files->dialog,
 		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
-		/* XmNresizePolicy, XmRESIZE_NONE, */
 		NULL);
   
-  rowcol = XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, dialog,
+  rowcol = XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, 
+			    deletion_files->dialog,
 			    XmNorientation, XmVERTICAL,
 			    XmNentryAlignment, XmALIGNMENT_CENTER,
 			    NULL);
@@ -221,11 +216,11 @@ void ok_deletion(Widget w, XtPointer client_data, XtPointer call_data)
   /* Add each file selected to dir or filelist depending on whether it is a
    * dir or a file
    */
-  for(i = 0; i < deletion_files.position_count; i++) {
-    index = deletion_files.position_list[i] - 1;
-    filename = deletion_files.files[index].filename;
+  for(i = 0; i < deletion_files->position_count; i++) {
+    index = deletion_files->position_list[i] - 1;
+    filename = deletion_files->files[index].filename;
     file = XmStringCreateLocalized(filename);
-    if(deletion_files.files[index].directory)
+    if(deletion_files->files[index].directory)
       XmListAddItem(dir_list, file, 0);
     else
       XmListAddItem(file_list, file, 0);
@@ -253,40 +248,45 @@ void ok_deletion(Widget w, XtPointer client_data, XtPointer call_data)
 				   XmNtopAttachment, XmATTACH_FORM,
 				   NULL);
  
-  XtAddCallback(ok, XmNactivateCallback, perform_deletion, dialog);
-  XtAddCallback(cancel, XmNactivateCallback, button_pre_cancel_deletion, 
-		dialog);
+  XtAddCallback(ok, XmNactivateCallback, perform_deletion, deletion_files);
+  XtAddCallback(cancel, XmNactivateCallback, cancel_deletion, 
+		deletion_files);
 
   XtManageChild(buttons);
   XtManageChild(rowcol);
-  XtManageChild(dialog);
+  XtManageChild(deletion_files->dialog);
 }
 /***************************************************************************/
   
 void stop_deletion(Widget w, XtPointer client_data, XtPointer call_data)
 {
+  DelFType *dfs = (DelFType *) client_data;
+
   /* The variable should be checked occasionally */
-  deletion_files.cancel_deletion = 1;
+  dfs->cancel_deletion = 1;
 }
 /***************************************************************************/
 
 void perform_deletion(Widget w, XtPointer client_data, XtPointer call_data)
 {
-  Widget dialog = (Widget) client_data;
-  Widget main_w = deletion_files.main_cb->main_window;
+  DelFType *dfs = (DelFType *) client_data;
+  Widget main_w;
   Widget info;
   int i, index, dir_error = 0;
   char *filename;
   XmString text;
   char templine[LINE_MAX];
+  Main_CB *main_cb;
 
+  main_cb = dfs->main_cb;
+  main_w = main_cb->main_window;
   /* Kill the confirmation dialog */
-  XtDestroyWidget(dialog);
+  XtDestroyWidget(dfs->dialog);
   
   /* Setup a dialog to show what files are being deleted in real-time */
-  info = XmCreateInformationDialog(deletion_files.main_cb->main_window, 
+  info = XmCreateInformationDialog(main_w, 
 				   "Deleting", NULL, 0);
-  deletion_files.dialog = info;
+  dfs->dialog = info;
   
   text = XmStringCreateLocalized("Currently removing...");
 
@@ -298,23 +298,23 @@ void perform_deletion(Widget w, XtPointer client_data, XtPointer call_data)
 		NULL);
   XmStringFree(text);
 
-  XtAddCallback(info, XmNcancelCallback, stop_deletion, info);
+  XtAddCallback(info, XmNcancelCallback, stop_deletion, dfs);
   XtUnmanageChild(XmMessageBoxGetChild(info, XmDIALOG_OK_BUTTON));
   XtUnmanageChild(XmMessageBoxGetChild(info, XmDIALOG_HELP_BUTTON));
   
   /* Provide a dummy filename for good sizing of dialog */
-  deletion_files.file_label = 
+  dfs->file_label = 
     XtVaCreateManagedWidget("...", xmLabelWidgetClass, info, NULL);
   
   XtManageChild(info);
-  refresh_screen(deletion_files.main_cb);
+  refresh_screen(dfs->main_cb);
 
   /* Actual gist of deletion */
-  for(i = 0; i < deletion_files.position_count; i++) {
+  for(i = 0; i < dfs->position_count; i++) {
     /* Check for cancel by user through each iteration */
-    CheckForInterrupt(deletion_files.main_cb, info);
+    CheckForInterrupt(main_cb, info);
 
-    if(deletion_files.cancel_deletion) {
+    if(dfs->cancel_deletion) {
       /* User cancelled */
       info_dialog(main_w, "Deletion was cancelled.");
 
@@ -324,16 +324,16 @@ void perform_deletion(Widget w, XtPointer client_data, XtPointer call_data)
       dir_error = 1;
       break;
     }
-    index = deletion_files.position_list[i] - 1;
-    filename = deletion_files.files[index].filename;
+    index = dfs->position_list[i] - 1;
+    filename = dfs->files[index].filename;
 
-    if(deletion_files.files[index].directory) {
+    if(dfs->files[index].directory) {
       /* Delete everything in this directory */
-      if(delete_subdir(filename)) {
-	update_delete_dialog(filename, 1);
+      if(delete_subdir(filename, dfs)) {
+	update_delete_dialog(filename, 1, dfs);
 
 	/* Now remove the directory */
-	if(!section_remove_dir(filename)) {
+	if(!section_remove_dir(filename, dfs)) {
 	  sprintf(templine, "Could not remove directory %s.", filename);
 	  error_dialog(main_w, templine);
 	  break;
@@ -345,8 +345,8 @@ void perform_deletion(Widget w, XtPointer client_data, XtPointer call_data)
       }
     }
     else {
-      update_delete_dialog(filename, 0);
-      if(!section_delete_file(filename)) {
+      update_delete_dialog(filename, 0, dfs);
+      if(!section_delete_file(filename, dfs)) {
 	sprintf(templine, "Could not remove file %s.", filename);
 	error_dialog(main_w, templine);
 	/* should not be out of sync but who knows */
@@ -355,90 +355,92 @@ void perform_deletion(Widget w, XtPointer client_data, XtPointer call_data)
       }
     }
   }
-  free(deletion_files.position_list);
 
-  if(!deletion_files.cancel_deletion)
+  if(!dfs->cancel_deletion)
     /* user did not stop deletiong (dialog still up) kill it */
     XtDestroyWidget(info);
  
   if(dir_error) {
     /* Reset the CWD label */
-    if(deletion_files.remote)
-      set_remote_dir_label(deletion_files.main_cb);
+    if(dfs->remote)
+      set_remote_dir_label(main_cb);
     else
-      set_local_dir_label(deletion_files.main_cb);
+      set_local_dir_label(main_cb);
   }
 
   /* always refresh dirlist */
-  if(deletion_files.remote)
-    refresh_remote_dirlist(deletion_files.main_cb);
+  if(dfs->remote)
+    refresh_remote_dirlist(main_cb);
   else
-    refresh_local_dirlist(deletion_files.main_cb, ".");
+    refresh_local_dirlist(main_cb, ".");
+
+  free(dfs->position_list);
+  free(dfs);
 }
 /***************************************************************************/
 
-int delete_subdir(char *filename)
+int delete_subdir(char *filename, DelFType *dfs)
 {
   FileList *files;
   DirState state = { LONG, NAME, ASCENDING };
   int num_files, i;
   char templine[LINE_MAX];
 
-  if(!section_change_dir(filename)) {
+  if(!section_change_dir(filename, dfs)) {
     sprintf(templine, "Could not change into %s directory %s.", 
-	    deletion_files.remote ? "remote" : "local",
+	    dfs->remote ? "remote" : "local",
 	    filename);
-    error_dialog(deletion_files.main_cb->main_window, templine);
+    error_dialog(dfs->main_cb->main_window, templine);
     return 0;
   }
 
-  refresh_screen(deletion_files.main_cb);
+  refresh_screen(dfs->main_cb);
 
   /* Get files in current directory */
-  files = deletion_files.remote ? 
+  files = dfs->remote ? 
     generate_remote_filelist(&num_files, state, 
-			     deletion_files.main_cb->current_state.
+			     dfs->main_cb->current_state.
 			     connection_state.site_info.system_type) :
     generate_local_filelist(".", &num_files, state); 
   
   if(!files) {
     /* no files in dir? */
-    section_change_dir(".."); /* might be prob */
+    section_change_dir("..", dfs); /* might be prob */
     return 0;
   }
 
   /* Process filelist (recurse if necessary) similar to above */
   for(i = 2; i < num_files; i++) {
-    CheckForInterrupt(deletion_files.main_cb, deletion_files.dialog);
-    if(deletion_files.cancel_deletion) {
+    CheckForInterrupt(dfs->main_cb, dfs->dialog);
+    if(dfs->cancel_deletion) {
       destroy_filelist(&files, num_files);
       return 0;
     }
     if(files[i].directory) {
-      if(!delete_subdir(files[i].filename)) {
+      if(!delete_subdir(files[i].filename, dfs)) {
 	destroy_filelist(&files, num_files);
 	return 0;
       }
-      update_delete_dialog(files[i].filename, 1);
-      if(!section_remove_dir(files[i].filename)) {
+      update_delete_dialog(files[i].filename, 1, dfs);
+      if(!section_remove_dir(files[i].filename, dfs)) {
 	sprintf(templine, "Could not remove directory %s.", files[i].filename);
-	error_dialog(deletion_files.main_cb->main_window, templine);
+	error_dialog(dfs->main_cb->main_window, templine);
 	destroy_filelist(&files, num_files);
 	return 0;
       }
     }
     else {
-      update_delete_dialog(files[i].filename, 0);
-      if(!section_delete_file(files[i].filename)) {
+      update_delete_dialog(files[i].filename, 0, dfs);
+      if(!section_delete_file(files[i].filename, dfs)) {
 	sprintf(templine, "Could not remove file %s.", files[i].filename);
-	error_dialog(deletion_files.main_cb->main_window, templine);
+	error_dialog(dfs->main_cb->main_window, templine);
 	destroy_filelist(&files, num_files);
 	return 0;
       }
     }
   }
   destroy_filelist(&files, num_files);
-  section_change_dir("..");
+  section_change_dir("..", dfs);
 
   return 1;
 }
@@ -524,7 +526,7 @@ void process_local_mkdir(Widget w, XtPointer client_data,
 }
 /***************************************************************************/
 
-void update_delete_dialog(char *filename, int dir)
+void update_delete_dialog(char *filename, int dir, DelFType *dfs)
 {
   XmString text;
   int length;
@@ -539,33 +541,33 @@ void update_delete_dialog(char *filename, int dir)
     temp = filename;
 
   text = XmStringCreateLocalized(temp);
-  XtVaSetValues(deletion_files.file_label, XmNlabelString, text, NULL);
-  refresh_screen(deletion_files.main_cb);
+  XtVaSetValues(dfs->file_label, XmNlabelString, text, NULL);
+  refresh_screen(dfs->main_cb);
   XmStringFree(text);
 }
 /***************************************************************************/
 
-int section_change_dir(char *filename)
+int section_change_dir(char *filename, DelFType *dfs)
 {
-  if(deletion_files.remote)
+  if(dfs->remote)
     return(change_remote_dir(filename));
   else
     return(change_dir(filename)); 
 }
 /***************************************************************************/
 
-int section_remove_dir(char *filename)
+int section_remove_dir(char *filename, DelFType *dfs)
 {
-  if(deletion_files.remote)
+  if(dfs->remote)
     return(remove_remote_dir(filename));
   else 
     return(remove_local_dir(filename));
 }
 /***************************************************************************/
     
-int section_delete_file(char *filename)
+int section_delete_file(char *filename, DelFType *dfs)
 {
-  if(deletion_files.remote)
+  if(dfs->remote)
     return(delete_remote_file(filename));
   else 
     return(delete_local_file(filename));
