@@ -16,6 +16,7 @@
 #include <Xm/SelectioB.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <malloc.h>
 #include "xmftp.h"
 #include "../program/site_db/site_db.h"
 #include "../program/misc.h"
@@ -32,7 +33,8 @@
 
 typedef struct {
   Widget dialog, site, login, password, dir, comment, sitelist;
-  Widget b_edit, b_delete, b_connect, b_add; /* buttons */
+  Widget b_edit, b_delete, b_connect, b_add, b_addcurr; /* buttons */
+  Main_CB *main_cb;
 } SiteDispInfo;
 
 typedef struct {
@@ -40,10 +42,9 @@ typedef struct {
     portfield, portlabel, stdport, nstdport;
   PassField *passfield;
   Widget dialog;
+  SiteDispInfo *disp_info;
 } SiteAddInfo;
 
-SiteDispInfo disp_info;
-
 /***************************************************************************/
 /* Prompt for a password if password field blank and trying to connect
  * User does not have to store passwd in sitelist file
@@ -63,7 +64,7 @@ void login_field_changed(Widget w, XtPointer client_data,
 void set_port(Widget w, XtPointer client_data, XtPointer call_data);
 
 /* Clear the fields of dialog */
-void clear_info(void);
+void clear_info(SiteDispInfo *disp_info);
 
 /* Popup the dialog common to Add and Delete operations 
  * operations is 0 for Add, 1 for Edit
@@ -87,13 +88,14 @@ void delete_site(Widget w, XtPointer client_data, XtPointer call_data);
 void do_addition(Widget w, XtPointer client_data, XtPointer call_data);
 
 /* Setup the site information window/frame */
-void addto_site_info(Widget w);
+void addto_site_info(Widget w, SiteDispInfo *disp_info);
 
 /* Load the user's sites into memory */
-void load_sites(void);
+void load_sites(SiteDispInfo *disp_info);
+
 
 /* Setup callbacks on site manager dialog */
-void setup_cbs(Main_CB *main_cb);
+void setup_cbs(SiteDispInfo *disp_info);
 
 /* Update display (when site is deleted/modified) */
 void modify_display(Widget w, XtPointer client_data, XtPointer call_data);
@@ -105,23 +107,31 @@ void add_new_site(Widget w, XtPointer client_data, XtPointer call_data);
 void edit_site(Widget w, XtPointer client_data, XtPointer call_data);
 
 /* Put buttons into the site manager (layout) */
-void add_buttons(Widget w);
+void add_buttons(Widget w, SiteDispInfo *disp_info);
 
 /* Connect to the selected site */
 void connect_from_site_mgr(Widget w, XtPointer client_data, XtPointer 
 			   call_data);
 
+void add_current_site(Widget w, XtPointer client_data, XtPointer call_data);
+
 /***************************************************************************/
 
 void site_manager(Widget w, XtPointer client_data, XtPointer call_data)
 {
   Main_CB *main_cb = (Main_CB *) client_data;
-  Widget dialog, slist, frame, s_info;
+  Widget dialog, frame, s_info;
   Arg args[15];
   int n = 0;
+  SiteDispInfo *disp_info = NULL;
+
+  disp_info = (SiteDispInfo *) malloc(sizeof(SiteDispInfo));
+  disp_info->main_cb = main_cb;
+
+  dialog = XmCreateFormDialog(main_cb->main_window, "Site Manager",
+			      NULL, 0);
+  disp_info->dialog = dialog;
 
-  dialog = XmCreateFormDialog(main_cb->main_window, "Site Manager", NULL, 0);
-  
   XtVaSetValues(dialog,
 		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
 		XmNwidth, 400,
@@ -136,8 +146,9 @@ void site_manager(Widget w, XtPointer client_data, XtPointer call_data)
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
   XtSetArg(args[n], XmNrightPosition, 30); n++;
-  slist = XmCreateScrolledList(dialog, "slist", args, n);
-  XtManageChild(slist);
+  disp_info->sitelist = XmCreateScrolledList(dialog, "slist", args,
+					     n);
+  XtManageChild(disp_info->sitelist);
 
   frame = XtVaCreateWidget("frame1", xmFrameWidgetClass, dialog,
 			   XmNtopAttachment, XmATTACH_FORM,
@@ -155,21 +166,20 @@ void site_manager(Widget w, XtPointer client_data, XtPointer call_data)
 			  XmNchildVerticalAlignment, XmALIGNMENT_CENTER,
 			  NULL);
   s_info = XtVaCreateWidget("form", xmFormWidgetClass, frame, NULL);
-  addto_site_info(s_info);
-  disp_info.sitelist = slist;
-  disp_info.dialog = dialog;
+  addto_site_info(s_info, disp_info);
 
   XtManageChild(s_info);
   XtManageChild(frame);
 
-  load_sites();
+  load_sites(disp_info);
 
-  add_buttons(dialog);
+  add_buttons(dialog, disp_info);
 
-  setup_cbs(main_cb);
-  XtSetSensitive(disp_info.b_edit, False);
-  XtSetSensitive(disp_info.b_delete, False);
-  XtSetSensitive(disp_info.b_connect, False);
+  setup_cbs(disp_info);
+
+  XtSetSensitive(disp_info->b_edit, False);
+  XtSetSensitive(disp_info->b_delete, False);
+  XtSetSensitive(disp_info->b_connect, False);
 
   XtManageChild(dialog);
 }
@@ -381,22 +391,27 @@ Widget do_common_add_mod_dialog(Widget main_window, int operation,
 
 void add_new_site(Widget w, XtPointer client_data, XtPointer call_data)
 {
-  Main_CB *main_cb = (Main_CB *) client_data;
+  SiteDispInfo *disp_info = (SiteDispInfo *) client_data;
+  Main_CB *main_cb = disp_info->main_cb;
   Widget dialog;
-  static SiteAddInfo add_info;
+  SiteAddInfo *add_info;
   Widget b_add, b_cancel;
 
+  add_info = (SiteAddInfo *) malloc(sizeof(SiteAddInfo));
+  add_info->disp_info = disp_info;
+
   dialog = do_common_add_mod_dialog(main_cb->main_window, 0,
-				    &b_add, &b_cancel, &add_info);
+				    &b_add, &b_cancel, add_info);
 
   XtVaSetValues(b_add, XmNuserData, NULL, NULL);
+  XtVaSetValues(b_cancel, XmNuserData, NULL, NULL);
 
-  XmToggleButtonSetState(add_info.stdport, True, True);
+  XmToggleButtonSetState(add_info->stdport, True, True);
 
-  XtAddCallback(b_add, XmNactivateCallback, do_addition, &add_info);
-  XtAddCallback(b_cancel, XmNactivateCallback, cancelled_add, &add_info);
-  XtAddCallback(add_info.loginfield, XmNvalueChangedCallback, 
-		login_field_changed, add_info.passfield);
+  XtAddCallback(b_add, XmNactivateCallback, do_addition, add_info);
+  XtAddCallback(b_cancel, XmNactivateCallback, cancelled_add, add_info);
+  XtAddCallback(add_info->loginfield, XmNvalueChangedCallback, 
+		login_field_changed, add_info->passfield);
   XtManageChild(dialog);
 
 }
@@ -422,66 +437,66 @@ void login_field_changed(Widget w, XtPointer client_data, XtPointer call_data)
 
 void edit_site(Widget w, XtPointer client_data, XtPointer call_data)
 {
-  Main_CB *main_cb = (Main_CB *) client_data;
+  SiteDispInfo *disp_info = (SiteDispInfo *) client_data;
+  Main_CB *main_cb = disp_info->main_cb;
   Widget dialog;
-  static SiteAddInfo add_info;
+  SiteAddInfo *add_info;
   Widget b_add, b_cancel;
   int *position_list;
   int position_count;
   SiteInfo site;
   char *text;
   XmString *items;
-  static char directory[FILENAME_MAX];
+  char *directory;
 
-  dialog = do_common_add_mod_dialog(main_cb->main_window, 0,
-				    &b_add, &b_cancel, &add_info);
-
-  if(XmListGetSelectedPos(disp_info.sitelist, &position_list, &position_count)
+  if(XmListGetSelectedPos(disp_info->sitelist, &position_list, &position_count)
      == False)
     return;
 
   free(position_list);
 
   /* Get the selected site information */
-  XtVaGetValues(disp_info.sitelist, XmNselectedItems, &items, NULL);
+  XtVaGetValues(disp_info->sitelist, XmNselectedItems, &items, NULL);
 
   XmStringGetLtoR(*items, XmFONTLIST_DEFAULT_TAG, &text);
 
   if(!SiteInfoList_find_site(text, &site))
     return;
 
+  add_info = (SiteAddInfo *) malloc(sizeof(SiteAddInfo));
+  add_info->disp_info = disp_info;
+
   dialog = do_common_add_mod_dialog(main_cb->main_window, 1,
-				    &b_add, &b_cancel, &add_info);
+				    &b_add, &b_cancel, add_info);
   
   /* Modify the fields */
-  XmTextFieldSetString(add_info.hostfield, site.hostname);
-  XmTextFieldSetString(add_info.loginfield, site.login);
-  /*
-  XmTextFieldSetString(add_info.passfield, site.password);
-  */
-  if(!strcmp(site.login, "anonymous"))
-    PField_SetMode(add_info.passfield, 0);
+  XmTextFieldSetString(add_info->hostfield, site.hostname);
+  XmTextFieldSetString(add_info->loginfield, site.login);
 
-  PField_SetString(add_info.passfield, site.password);
-  XmTextFieldSetString(add_info.commfield, site.comment);
-  XmTextFieldSetString(add_info.nickfield, site.nickname);
+  if(!strcmp(site.login, "anonymous"))
+    PField_SetMode(add_info->passfield, 0);
+
+  PField_SetString(add_info->passfield, site.password);
+  XmTextFieldSetString(add_info->commfield, site.comment);
+  XmTextFieldSetString(add_info->nickfield, site.nickname);
 
   if(*(site.port)) {
-    XmToggleButtonSetState(add_info.nstdport, True, True);
-    XmTextFieldSetString(add_info.portfield, site.port);
+    XmToggleButtonSetState(add_info->nstdport, True, True);
+    XmTextFieldSetString(add_info->portfield, site.port);
   }
   else
-    XmToggleButtonSetState(add_info.stdport, True, True);
+    XmToggleButtonSetState(add_info->stdport, True, True);
     
-
+  directory = (char *) malloc(strlen(site.directory) + 1);
   strcpy(directory, site.directory);
-  XtVaSetValues(b_add, XmNuserData, &directory, NULL);
+  XtVaSetValues(b_add, XmNuserData, directory, NULL);
+  XtVaSetValues(b_cancel, XmNuserData, directory, NULL);
 
   /* do_addition will delete a site (if exists) and add it again */
-  XtAddCallback(b_add, XmNactivateCallback, do_addition, &add_info);
-  XtAddCallback(b_cancel, XmNactivateCallback, cancelled_add, &add_info);
-  XtAddCallback(add_info.loginfield, XmNvalueChangedCallback, 
-		login_field_changed, add_info.passfield);
+  XtAddCallback(b_add, XmNactivateCallback, do_addition, add_info);
+  XtAddCallback(b_cancel, XmNactivateCallback, cancelled_add, add_info);
+  XtAddCallback(add_info->loginfield, XmNvalueChangedCallback, 
+		login_field_changed, add_info->passfield);
 		
   XtManageChild(dialog);
 }
@@ -490,6 +505,7 @@ void edit_site(Widget w, XtPointer client_data, XtPointer call_data)
 void do_addition(Widget w, XtPointer client_data, XtPointer call_data)
 {
   SiteAddInfo *add_info = (SiteAddInfo *) client_data;
+  SiteDispInfo *disp_info = add_info->disp_info;
   char *file;
   char *host, *login, *pass, *comm, *nick, *directory, *port;
   char *anon_login = "anonymous";
@@ -502,9 +518,7 @@ void do_addition(Widget w, XtPointer client_data, XtPointer call_data)
   /* Get the information user entered */
   host = XmTextFieldGetString(add_info->hostfield);
   login = XmTextFieldGetString(add_info->loginfield);
-  /*
-  pass = XmTextFieldGetString(add_info->passfield);
-  */
+
   pass = PField_GetString(add_info->passfield);
   comm = XmTextFieldGetString(add_info->commfield);
   nick = XmTextFieldGetString(add_info->nickfield);
@@ -531,8 +545,12 @@ void do_addition(Widget w, XtPointer client_data, XtPointer call_data)
   strcpy(site.hostname, host);
   XtVaGetValues(w, XmNuserData, &directory, NULL);
   /* Save last directory if needed */
-  if(directory)
+  if(directory) {
     strcpy(site.directory, directory);
+    free(directory);
+    XtVaSetValues(w, XmNuserData, NULL, NULL);
+    directory = NULL;
+  }
   else
     *(site.directory) = '\0';
 
@@ -547,57 +565,82 @@ void do_addition(Widget w, XtPointer client_data, XtPointer call_data)
   SiteInfoList_delete_site(site.nickname, NULL);
   SiteInfoList_add_site(&site);
   SiteInfoList_save_sites(file);
-  load_sites();
+  load_sites(disp_info);
   
   PField_Destroy(add_info->passfield);
   XtPopdown(XtParent(add_info->dialog));
   XtDestroyWidget(XtParent(add_info->dialog));
 
   string = XmStringCreateLocalized(site.nickname);
-  XmListSelectItem(disp_info.sitelist, string, True);
+  XmListSelectItem(disp_info->sitelist, string, True);
+  free(add_info);
   XmStringFree(string);
 }
 /***************************************************************************/
 
-void add_buttons(Widget w)
+void add_buttons(Widget w, SiteDispInfo *disp_info)
 {
-  Widget add, delete, edit, connect, cancel;
+  Widget cancel;
+  Main_CB *main_cb = disp_info->main_cb;
 
-  add = XtVaCreateManagedWidget("Add Site", xmPushButtonWidgetClass, w,
-				XmNleftAttachment, XmATTACH_POSITION,
-				XmNleftPosition, 30,
-				XmNrightAttachment, XmATTACH_POSITION,
-				XmNrightPosition, 50,
-				XmNtopAttachment, XmATTACH_POSITION,
-				XmNtopPosition, 80,
-				NULL);
+  if(IsConnected(main_cb) && !IsConnectedFromSiteMgr(main_cb)) {
+    /* add a button to add current site connected to */
+    disp_info->b_addcurr = 
+      XtVaCreateManagedWidget("Add current site", 
+			      xmPushButtonWidgetClass, w,
+			      XmNleftAttachment, XmATTACH_POSITION,
+			      XmNleftPosition, 45,
+			      XmNrightAttachment, XmATTACH_POSITION,
+			      XmNrightPosition, 85,
+			      XmNtopAttachment, XmATTACH_POSITION,
+			      XmNtopPosition, 70,
+			      NULL);
+  }
+  else
+    disp_info->b_addcurr = NULL;
 
-  delete = XtVaCreateManagedWidget("Delete Site", xmPushButtonWidgetClass, w,
-				   XmNleftAttachment, XmATTACH_POSITION,
-				   XmNleftPosition, 55,
-				   XmNrightAttachment, XmATTACH_POSITION,
-				   XmNrightPosition, 75,
-				   XmNtopAttachment, XmATTACH_POSITION,
-				   XmNtopPosition, 80,
-				   NULL);
+  disp_info->b_add = XtVaCreateManagedWidget("Add Site",
+					     xmPushButtonWidgetClass, w,
+					     XmNleftAttachment, 
+					     XmATTACH_POSITION,
+					     XmNleftPosition, 30,
+					     XmNrightAttachment, 
+					     XmATTACH_POSITION,
+					     XmNrightPosition, 50,
+					     XmNtopAttachment, 
+					     XmATTACH_POSITION,
+					     XmNtopPosition, 80,
+					     NULL);
 
-  edit = XtVaCreateManagedWidget("Edit Site", xmPushButtonWidgetClass, w,
-				 XmNleftAttachment, XmATTACH_POSITION,
-				 XmNleftPosition, 80,
-				 XmNrightAttachment, XmATTACH_POSITION,
-				 XmNrightPosition, 100,
-				 XmNtopAttachment, XmATTACH_POSITION,
-				 XmNtopPosition, 80,
-				 NULL);
+  disp_info->b_delete = 
+    XtVaCreateManagedWidget("Delete Site", xmPushButtonWidgetClass, w,
+			    XmNleftAttachment, XmATTACH_POSITION,
+			    XmNleftPosition, 55,
+			    XmNrightAttachment, XmATTACH_POSITION,
+			    XmNrightPosition, 75,
+			    XmNtopAttachment, XmATTACH_POSITION,
+			    XmNtopPosition, 80,
+			    NULL);
+  
+  disp_info->b_edit = 
+    XtVaCreateManagedWidget("Edit Site", xmPushButtonWidgetClass, w,
+			    XmNleftAttachment, XmATTACH_POSITION,
+			    XmNleftPosition, 80,
+			    XmNrightAttachment, XmATTACH_POSITION,
+			    XmNrightPosition, 100,
+			    XmNtopAttachment, XmATTACH_POSITION,
+			    XmNtopPosition, 80,
+			    NULL);
 
-  connect = XtVaCreateManagedWidget("Connect", xmPushButtonWidgetClass, w,
-				    XmNleftAttachment, XmATTACH_POSITION,
-				    XmNleftPosition, 42,
-				    XmNrightAttachment, XmATTACH_POSITION,
-				    XmNrightPosition, 62,
-				    XmNtopAttachment, XmATTACH_POSITION,
-				    XmNtopPosition, 90,
-				    NULL);
+  disp_info->b_connect = 
+    XtVaCreateManagedWidget("Connect", xmPushButtonWidgetClass, w,
+			    XmNleftAttachment, XmATTACH_POSITION,
+			    XmNleftPosition, 42,
+			    XmNrightAttachment, XmATTACH_POSITION,
+			    XmNrightPosition, 62,
+			    XmNtopAttachment, XmATTACH_POSITION,
+			    XmNtopPosition, 90,
+			    NULL);
 
   cancel = XtVaCreateManagedWidget("Cancel", xmPushButtonWidgetClass, w,
 				   XmNleftAttachment, XmATTACH_POSITION,
@@ -609,36 +652,33 @@ void add_buttons(Widget w)
 				   NULL);
 
   XtAddCallback(cancel, XmNactivateCallback, cancelled, 
-		disp_info.dialog);
-
-  disp_info.b_add = add;
-  disp_info.b_edit = edit;
-  disp_info.b_delete = delete;
-  disp_info.b_connect = connect;
-  
+		disp_info);
 }
 /***************************************************************************/
 
-void setup_cbs(Main_CB *main_cb)
+void setup_cbs(SiteDispInfo *disp_info)
 {
-  XtAddCallback(disp_info.sitelist, XmNbrowseSelectionCallback, 
-		modify_display, NULL);
+  XtAddCallback(disp_info->sitelist, XmNbrowseSelectionCallback, 
+		modify_display, disp_info);
   
-  XtAddCallback(disp_info.b_connect, XmNactivateCallback,
-		connect_from_site_mgr, main_cb);
+  XtAddCallback(disp_info->b_connect, XmNactivateCallback,
+		connect_from_site_mgr, disp_info);
 
-  XtAddCallback(disp_info.sitelist, XmNdefaultActionCallback,
-		connect_from_site_mgr, main_cb);
+  XtAddCallback(disp_info->sitelist, XmNdefaultActionCallback,
+		connect_from_site_mgr, disp_info);
 
-  XtAddCallback(disp_info.b_add, XmNactivateCallback,
-		add_new_site, main_cb);
+  XtAddCallback(disp_info->b_add, XmNactivateCallback,
+		add_new_site, disp_info);
 
-  XtAddCallback(disp_info.b_edit, XmNactivateCallback,
-		edit_site, main_cb);
+  XtAddCallback(disp_info->b_edit, XmNactivateCallback,
+		edit_site, disp_info);
 
-  XtAddCallback(disp_info.b_delete, XmNactivateCallback,
-		delete_site, main_cb);
+  XtAddCallback(disp_info->b_delete, XmNactivateCallback,
+		delete_site, disp_info);
   
+  if(disp_info->b_addcurr) 
+    XtAddCallback(disp_info->b_addcurr, XmNactivateCallback,
+		  add_current_site, disp_info);
 }
 /***************************************************************************/
 
@@ -646,61 +686,69 @@ void cancelled_add(Widget w, XtPointer client_data, XtPointer call_data)
 {
   SiteAddInfo *add_info = (SiteAddInfo *) client_data;
   Widget blah;
+  char *dir = NULL;
+
+  XtVaGetValues(w, XmNuserData, &dir, NULL);
+  
+  if(dir)
+    free(dir);
 
   blah = add_info->dialog;
   PField_Destroy(add_info->passfield);
 
   XtPopdown(XtParent(blah));
   XtDestroyWidget(XtParent(blah));
+  free(add_info);
 }
 /***************************************************************************/
 
 void cancelled(Widget w, XtPointer client_data, XtPointer call_data)
 {
-  Widget blah = (Widget) client_data;
+  SiteDispInfo *disp_info = (SiteDispInfo *) client_data;
+  Widget blah = disp_info->dialog;
 
   XtPopdown(XtParent(blah));
   XtDestroyWidget(XtParent(blah));
   SiteInfoList_destroy_list();
+  free(disp_info);
 }
 /***************************************************************************/
 
-void clear_info(void)
+void clear_info(SiteDispInfo *disp_info)
 {
 
-  XtVaSetValues(disp_info.site, XtVaTypedArg, XmNlabelString, XmRString, "",
+  XtVaSetValues(disp_info->site, XtVaTypedArg, XmNlabelString, XmRString, "",
 		1, NULL);
-  XtVaSetValues(disp_info.login, XtVaTypedArg, XmNlabelString, XmRString, "",
+  XtVaSetValues(disp_info->login, XtVaTypedArg, XmNlabelString, XmRString, "",
 		1, NULL);
-  XtVaSetValues(disp_info.password, XtVaTypedArg, XmNlabelString, XmRString, 
+  XtVaSetValues(disp_info->password, XtVaTypedArg, XmNlabelString, XmRString, 
 		"", 1, NULL);
-  XtVaSetValues(disp_info.dir, XtVaTypedArg, XmNlabelString, XmRString, "",
+  XtVaSetValues(disp_info->dir, XtVaTypedArg, XmNlabelString, XmRString, "",
 		1, NULL);
-  XmTextSetString(disp_info.comment, "");
+  XmTextSetString(disp_info->comment, "");
 }
 /***************************************************************************/
 
 void delete_site(Widget w, XtPointer client_data, XtPointer call_data)
 {
-  int *position_list;
-  int position_count;
+  SiteDispInfo *disp_info = (SiteDispInfo *) client_data;
+  int *position_list, position_count;
   XmString *items;
   char *text;
   char *file;
 
-  if(XmListGetSelectedPos(disp_info.sitelist, &position_list, &position_count)
+  if(XmListGetSelectedPos(disp_info->sitelist, &position_list, &position_count)
      == False) {
     error_dialog(w, "Please select a site to delete.");
     return;
   }
 
-  XtVaGetValues(disp_info.sitelist, XmNselectedItems, &items, NULL);
+  XtVaGetValues(disp_info->sitelist, XmNselectedItems, &items, NULL);
   
   XmStringGetLtoR(*items, XmFONTLIST_DEFAULT_TAG, &text);
 
-  if(!SiteInfoList_delete_site(text, NULL)) {
+  if(!SiteInfoList_delete_site(text, NULL))
     error_dialog(w, "Error deleting site!");
-  }
 
   file = get_sitelist_filename();
 
@@ -708,33 +756,32 @@ void delete_site(Widget w, XtPointer client_data, XtPointer call_data)
 
   free(text);
   free(position_list);
-  clear_info();
-  load_sites();
-  
+  clear_info(disp_info);
+  load_sites(disp_info);  
 }
 /***************************************************************************/
 
 void connect_from_site_mgr(Widget w, XtPointer client_data, 
 			   XtPointer call_data)
 {
-  Main_CB *main_cb = (Main_CB *) client_data;
-  int *position_list;
-  int position_count;
+  SiteDispInfo *disp_info = (SiteDispInfo *) client_data;
+  Main_CB *main_cb = disp_info->main_cb;
+  int *position_list, position_count;
   SiteInfo *site;
   XmString *items;
   char *text;
 
-  if(XmListGetSelectedPos(disp_info.sitelist, &position_list, &position_count)
+  if(XmListGetSelectedPos(disp_info->sitelist, &position_list, &position_count)
      == False)
     return;
 
-  XtVaGetValues(disp_info.sitelist, XmNselectedItems, &items, NULL);
+  XtVaGetValues(disp_info->sitelist, XmNselectedItems, &items, NULL);
 
   XmStringGetLtoR(*items, XmFONTLIST_DEFAULT_TAG, &text);
 
-  if(main_cb->current_state.connection_state.connect_from_site_mgr) {
+  if(IsConnectedFromSiteMgr(main_cb)) {
     do_kill_connection(main_cb);
-    load_sites();  /* for updating last directory */
+    load_sites(disp_info);  /* for updating last directory */
   }
   else
     do_kill_connection(main_cb);
@@ -746,8 +793,9 @@ void connect_from_site_mgr(Widget w, XtPointer client_data,
     return;
   }
 
-  XtPopdown(XtParent(disp_info.dialog));
-  XtDestroyWidget(disp_info.dialog);
+  XtPopdown(XtParent(disp_info->dialog));
+  XtDestroyWidget(disp_info->dialog);
+  free(disp_info);
 
   if(!(*(site->password))) {
     /* prompt for password */
@@ -834,9 +882,9 @@ int perform_login(Main_CB *main_cb, SiteInfo *site)
   return 0;
 }
 
-
 void modify_display(Widget w, XtPointer client_data, XtPointer call_data)
 {
+  SiteDispInfo *disp_info = (SiteDispInfo *) client_data;
   SiteInfo site;
   XmListCallbackStruct *cbs = (XmListCallbackStruct *) call_data;
   char *text;
@@ -849,33 +897,33 @@ void modify_display(Widget w, XtPointer client_data, XtPointer call_data)
   SiteInfoList_find_site(text, &site);
 
   string = XmStringCreateLocalized(site.hostname);
-  XtVaSetValues(disp_info.site, XmNlabelString, string, NULL);
+  XtVaSetValues(disp_info->site, XmNlabelString, string, NULL);
   XmStringFree(string);
 
   string = XmStringCreateLocalized(site.login);
-  XtVaSetValues(disp_info.login, XmNlabelString, string, NULL);
+  XtVaSetValues(disp_info->login, XmNlabelString, string, NULL);
   XmStringFree(string);
 
   string = XmStringCreateLocalized("*******");
-  XtVaSetValues(disp_info.password, XmNlabelString, string, NULL);
+  XtVaSetValues(disp_info->password, XmNlabelString, string, NULL);
   XmStringFree(string);
 
   string = XmStringCreateLocalized(site.directory);
-  XtVaSetValues(disp_info.dir, XmNlabelString, string, NULL);
+  XtVaSetValues(disp_info->dir, XmNlabelString, string, NULL);
   XmStringFree(string);
 
-  XmTextSetString(disp_info.comment, site.comment);
+  XmTextSetString(disp_info->comment, site.comment);
 
   free(text);
 
-  XtSetSensitive(disp_info.b_edit, True);
-  XtSetSensitive(disp_info.b_delete, True);
-  XtSetSensitive(disp_info.b_connect, True);
+  XtSetSensitive(disp_info->b_edit, True);
+  XtSetSensitive(disp_info->b_delete, True);
+  XtSetSensitive(disp_info->b_connect, True);
   
 }
 /***************************************************************************/
   
-void load_sites(void)
+void load_sites(SiteDispInfo *disp_info)
 {
   SiteInfo site;
   int num, i = 0;
@@ -884,7 +932,7 @@ void load_sites(void)
 
   config_file = get_sitelist_filename();
 
-  XmListDeleteAllItems(disp_info.sitelist);
+  XmListDeleteAllItems(disp_info->sitelist);
 
   if(!(num = SiteInfoList_read_list(config_file)))
     return;
@@ -904,7 +952,7 @@ void load_sites(void)
     i++;
   }
 
-  XmListAddItems(disp_info.sitelist, items, num, 0);
+  XmListAddItems(disp_info->sitelist, items, num, 0);
 
   for(i = 0; i < num; i++)
     XmStringFree(items[i]);
@@ -913,10 +961,9 @@ void load_sites(void)
 }
 /***************************************************************************/
 
-void addto_site_info(Widget w)
+void addto_site_info(Widget w, SiteDispInfo *disp_info)
 {
-  Widget site, login, password, dir, frame, form,
-    site_f, login_f, password_f, dir_f, comment;
+  Widget site, login, password, dir, frame, form;
 
   form = XtVaCreateWidget("form", xmFormWidgetClass, w,
 			  XmNtopAttachment, XmATTACH_FORM,
@@ -963,43 +1010,51 @@ void addto_site_info(Widget w)
 				XmNrightPosition, 35,
 				NULL);
 
-  site_f = XtVaCreateManagedWidget("", xmLabelWidgetClass, form, 
-				   XmNalignment, XmALIGNMENT_BEGINNING,
-				   XmNtopOffset, 5,
-				   XmNtopAttachment, XmATTACH_FORM,
-				   XmNleftAttachment, XmATTACH_WIDGET,
-				   XmNleftWidget, site,
-				   XmNrightAttachment, XmATTACH_FORM,
-				   NULL);
-  login_f = XtVaCreateManagedWidget("", xmLabelWidgetClass, form, 
-				    XmNalignment, XmALIGNMENT_BEGINNING,
-				    XmNtopOffset, 5,
-				    XmNtopAttachment, XmATTACH_WIDGET,
-				    XmNtopWidget, site_f,
-				    XmNleftAttachment, XmATTACH_WIDGET,
-				    XmNleftWidget, login,
-				    XmNrightAttachment, XmATTACH_FORM,
-				    NULL);
-  password_f = XtVaCreateManagedWidget("", xmLabelWidgetClass, form, 
-				       XmNalignment, XmALIGNMENT_BEGINNING,
-				       XmNtopOffset, 5,
-				       XmNtopAttachment, XmATTACH_WIDGET,
-				       XmNtopWidget, login_f,
-				       XmNleftAttachment, XmATTACH_WIDGET,
-				       XmNleftWidget, password,
-				       XmNrightAttachment, XmATTACH_FORM,
-				       NULL);
+  disp_info->site = XtVaCreateManagedWidget("", xmLabelWidgetClass, form, 
+					    XmNalignment, 
+					    XmALIGNMENT_BEGINNING,
+					    XmNtopOffset, 5,
+					    XmNtopAttachment, XmATTACH_FORM,
+					    XmNleftAttachment, XmATTACH_WIDGET,
+					    XmNleftWidget, site,
+					    XmNrightAttachment, XmATTACH_FORM,
+					    NULL);
+
+  disp_info->login = XtVaCreateManagedWidget("", xmLabelWidgetClass, form, 
+					     XmNalignment, 
+					     XmALIGNMENT_BEGINNING,
+					     XmNtopOffset, 5,
+					     XmNtopAttachment, XmATTACH_WIDGET,
+					     XmNtopWidget, disp_info->site,
+					     XmNleftAttachment,
+					     XmATTACH_WIDGET,
+					     XmNleftWidget, login,
+					     XmNrightAttachment, XmATTACH_FORM,
+					     NULL);
+  disp_info->password = XtVaCreateManagedWidget("", xmLabelWidgetClass, form, 
+						XmNalignment, 
+						XmALIGNMENT_BEGINNING,
+						XmNtopOffset, 5,
+						XmNtopAttachment, 
+						XmATTACH_WIDGET,
+						XmNtopWidget, disp_info->login,
+						XmNleftAttachment, 
+						XmATTACH_WIDGET,
+						XmNleftWidget, password,
+						XmNrightAttachment, 
+						XmATTACH_FORM,
+						NULL);
 
 
-  dir_f = XtVaCreateManagedWidget("", xmLabelWidgetClass, form, 
-				  XmNalignment, XmALIGNMENT_BEGINNING,
-				  XmNtopOffset, 5,
-				  XmNtopAttachment, XmATTACH_WIDGET,
-				  XmNtopWidget, password_f,
-				  XmNleftAttachment, XmATTACH_WIDGET,
-				  XmNleftWidget, dir,
-				  XmNrightAttachment, XmATTACH_FORM,
-				  NULL);
+  disp_info->dir = XtVaCreateManagedWidget("", xmLabelWidgetClass, form, 
+					   XmNalignment, XmALIGNMENT_BEGINNING,
+					   XmNtopOffset, 5,
+					   XmNtopAttachment, XmATTACH_WIDGET,
+					   XmNtopWidget, disp_info->password,
+					   XmNleftAttachment, XmATTACH_WIDGET,
+					   XmNleftWidget, dir,
+					   XmNrightAttachment, XmATTACH_FORM,
+					   NULL);
 
   XtManageChild(form);
   
@@ -1019,20 +1074,15 @@ void addto_site_info(Widget w)
 			  XmNchildVerticalAlignment, XmALIGNMENT_CENTER,
 			  NULL);
 
-  comment = XtVaCreateManagedWidget("text", xmTextWidgetClass, frame, NULL);
-  XtVaSetValues(comment, XmNeditable, False, 
+  disp_info->comment = XtVaCreateManagedWidget("text", xmTextWidgetClass, 
+					       frame, NULL);
+  XtVaSetValues(disp_info->comment, XmNeditable, False, 
 		XmNeditMode, XmMULTI_LINE_EDIT,
 		XmNwordWrap, True,
 		XmNcursorPositionVisible, False,
 		NULL);
 
   XtManageChild(frame);
-
-  disp_info.site = site_f;
-  disp_info.login = login_f;
-  disp_info.password = password_f;
-  disp_info.dir = dir_f;
-  disp_info.comment = comment;
 }
 /***************************************************************************/
 
@@ -1050,3 +1100,54 @@ void set_port(Widget w, XtPointer client_data, XtPointer call_data)
     XtSetSensitive(add_info->portfield, True);
   }
 }
+
+void add_current_site(Widget w, XtPointer client_data, XtPointer call_data)
+{
+  SiteDispInfo *disp_info = (SiteDispInfo *) client_data;
+  Main_CB *main_cb = disp_info->main_cb;
+  Widget dialog;
+  SiteAddInfo *add_info;
+  Widget b_add, b_cancel;
+  SiteInfo *site = 
+    &(main_cb->current_state.connection_state.site_info);
+  char *directory;
+  
+  add_info = (SiteAddInfo *) malloc(sizeof(SiteAddInfo));
+  add_info->disp_info = disp_info;
+  
+  dialog = do_common_add_mod_dialog(main_cb->main_window, 0,
+				    &b_add, &b_cancel, add_info);
+
+  XmTextFieldSetString(add_info->hostfield, site->hostname);
+  XmTextFieldSetString(add_info->loginfield, site->login);
+
+  if(!strcmp(site->login, "anonymous"))
+    PField_SetMode(add_info->passfield, 0);
+  
+  PField_SetString(add_info->passfield, site->password);
+  XmTextFieldSetString(add_info->commfield, "");
+  XmTextFieldSetString(add_info->nickfield, "");
+  
+  if(*(site->port)) { 
+    XmToggleButtonSetState(add_info->nstdport, True, True);
+    XmTextFieldSetString(add_info->portfield, site->port);
+  }
+  else
+    XmToggleButtonSetState(add_info->stdport, True, True);
+  
+  directory = (char *) malloc(strlen(site->directory) + 1);
+  strcpy(directory, site->directory);
+  XtVaSetValues(b_add, XmNuserData, directory, NULL);
+  XtVaSetValues(b_cancel, XmNuserData, directory, NULL);
+  
+  /* do_addition will delete a site (if exists) and add it again */
+  XtAddCallback(b_add, XmNactivateCallback, do_addition, add_info);
+  XtAddCallback(b_cancel, XmNactivateCallback, cancelled_add, add_info);
+  XtAddCallback(add_info->loginfield, XmNvalueChangedCallback, 
+		login_field_changed, add_info->passfield);
+		
+  XtManageChild(dialog);
+}
+
+
+
